import unittest
import numpy as np
from cryoloBM_tools.virons_prior import group_by_center

class MyTestCase(unittest.TestCase):
    def test_group_by_center(self):

        centers = np.zeros(shape=(2,3))
        centers[0, 0] = 5
        centers[0, 1] = 5
        centers[0, 2] = 5

        centers[1, 0] = 20
        centers[1, 1] = 20
        centers[1, 2] = 20

        picks = np.zeros(shape=(3,3))
        picks[0, 0] = 5 - 1
        picks[0, 1] = 5 - 1
        picks[0, 2] = 5 - 1

        picks[1, 0] = 5 + 1
        picks[1, 1] = 5 + 1
        picks[1, 2] = 5 + 1

        picks[2, 0] = 20 + 1
        picks[2, 1] = 20 + 1
        picks[2, 2] = 20 + 1

        groups = group_by_center(centers=centers,picks=picks,max_distance=10)
        np.testing.assert_array_equal([0,0,1], groups)



if __name__ == '__main__':
    unittest.main()



import glob

from cryoloBM.bmtool import BMTool
from argparse import ArgumentParser
import argparse
import cryolo.CoordsIO as io
from typing import List
from cryolo.utils import BoundBox
import numpy as np
import os


class cbox2json(BMTool):


    def get_command_name(self) -> str:
        '''
        :return: Command name
        '''
        return "cbox2json"

    def create_parser(self, parentparser: ArgumentParser) -> ArgumentParser:
        helptxt = "Converts .cbox files to EMAN .json files."
        parser = parentparser.add_parser(
            self.get_command_name(),
            help=helptxt,
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        )

        req = parser.add_argument_group(
            "Arguments",
            helptxt,
        )

        req.add_argument(
            "-i",
            "--input",
            required=True,
            nargs="+",
            help="Path to folder with .cbox files / path to single .cbox file ",
        )

        req.add_argument(
            "-n",
            "--name",
            required=True,
            help="Label for the picks",
        )

        req.add_argument(
            "-o",
            "--out",
            required=True,
            help="Output path",
        )
    def convert_to_eman_dict(self, boxes: List[BoundBox], name: str):
        eman_dict = {"boxes_3d": []}

        for box in boxes:
            c = 1.0

            if not np.isnan(box.c):
                c = box.c
            eman_dict["boxes_3d"].append([box.x+box.w/2,box.y+box.h/2,box.z,"cryolo",c,0])

        eman_dict["class_list"] = {
            "0": {
                "boxsize": int(box.w),
                "name": name
            }
        }

        return eman_dict


    def run(self, args):
        paths = []
        for pth in args.input:
            if os.path.isdir(pth):
                paths.extend(glob.glob(os.path.join(pth,"*.cbox")))
            elif os.path.isfile(pth):
                paths.append(pth)
        os.makedirs(args.out, exist_ok=True)
        for p in paths:
            boxes = io.read_cbox_boxfile(p)
            eman_dict = self.convert_to_eman_dict(boxes, args.name)
            new_filename = os.path.join(args.out, os.path.splitext(os.path.basename(p))[0] +".json")
            import json
            with open(new_filename, 'w') as fp:
                json.dump(eman_dict, fp, indent=1)



import typing

import pandas as pd
import tqdm

from cryoloBM.bmtool import BMTool
from argparse import ArgumentParser
import argparse
from typing import List
import numpy as np
from cryolo.utils import Filament, BoundBox
from cryolo.CoordsIO import read_star_file
from glob import glob
from pyStarDB import sp_pystardb as star
import tqdm
import os


class class_2d_extract(BMTool):

    def split_star(self, pth_star) -> typing.Dict[str,pd.DataFrame]:
        sfile = star.StarFile(pth_star)
        picks = sfile['particles']
        micrographs = picks['_rlnMicrographName']

        unique_mics = np.unique(micrographs)
        micrographs_np = micrographs.to_numpy()
        splitted_mics = {}
        for m in tqdm.tqdm(unique_mics):
            mask = micrographs_np==m
            splitted_mics[m] = picks.iloc[mask].copy()
        return splitted_mics

    def get_command_name(self) -> str:
        '''
        :return: Command name
        '''
        return "class2Dextract"

    def create_parser(self, parentparser: ArgumentParser) -> ArgumentParser:
        parser_autopick = parentparser.add_parser(
            self.get_command_name(),
            help="Extract coordinates from STAR files to make them usable for training with crYOLO.",
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        )

        autopick_required_group = parser_autopick.add_argument_group(
            "Required arguments",
            "Extract coordinates from STAR files to make them usable for training with crYOLO.",
        )

        autopick_required_group.add_argument(
            "-s",
            "--star",
            required=True,
            help="Path to star file",
        )

        autopick_required_group.add_argument(
            "-o",
            "--out",
            required=True,
            help="Output path",
        )

        autopick_required_group.add_argument(
            "-n",
            "--number",
            required=False,
            type=int,
            default=-1,
            help="Return only n most crowded micrographs.",
        )

    def run(self, args):

        star_pth = args.star
        os.makedirs(args.out,exist_ok=True)
        splitted_star = self.split_star(star_pth)
        splitted_star = dict(sorted(splitted_star.items(), key=lambda item: -1*len(item[1])))
        # use  tqdm here...
        for t_num, t in tqdm.tqdm(enumerate(splitted_star)):
            if args.number > 0 and t_num >= args.number:
                break

            mic_name = os.path.splitext(os.path.basename(t))[0]
            data = splitted_star[t][['_rlnCoordinateX','_rlnCoordinateY']]
            out_pth = os.path.join(args.out, f"{mic_name}.star")
            new_sfile = star.StarFile(out_pth)
            new_sfile.update('', data, loop=True)
            new_sfile.write_star_file(overwrite=True)
            print(f"Wrote {len(data)} particle to {out_pth}")





from cryoloBM.bmtool import BMTool
import argparse
from argparse import ArgumentParser
import os
from pyStarDB import sp_pystardb as star
import pandas as pd
from cryolo.CoordsIO import read_star_file
from typing import List
from cryolo.utils import BoundBox
class MakeCryoSparcStar(BMTool):

    def get_command_name(self) -> str:
        return "createCryoSparsStar"

    def create_parser(self, parentparser : ArgumentParser) -> ArgumentParser:
        parser_autopick = parentparser.add_parser(
            self.get_command_name(),
            help="Convert coordinates into a cryosparc STAR files.",
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        )

        autopick_required_group = parser_autopick.add_argument_group(
            "Required arguments",
            "Create STAR file that can be imported into cryoSPARC.",
        )

        autopick_required_group.add_argument(
            "-m",
            "--micrographs",
            required=True,
            nargs='+',
            help="Folder to micrographs",
        )

        autopick_required_group.add_argument(
            "-c",
            "--coordinates",
            required=True,
            nargs='+',
            help="Folder to star files from crYOLO",
        )

        autopick_required_group.add_argument(
            "-o",
            "--output",
            required=True,
            help="Output folder where to write the autopick.star.",
        )

    def run(self, args):
        micrographs = args.micrographs
        coordinates = args.coordinates


        micrographs_filename = [os.path.splitext(os.path.basename(m))[0] for m in micrographs]
        coordinates_filename = [os.path.splitext(os.path.basename(m))[0] for m in coordinates]

        micrographs_mapped = []
        coordinates_mapped = []
        for m_i,m in enumerate(micrographs_filename):
            try:
                index = coordinates_filename.index(m)

                micrographs_mapped.append(micrographs[m_i])
                coordinates_mapped.append(coordinates[index])
            except ValueError:
                print("Did not found matching coordinate file for ", micrographs[m_i])

        coords_x = []
        coords_y = []
        micrograph_names = []
        for c_i, coords_pth in enumerate(coordinates_mapped):
            coordinates: List[BoundBox] = read_star_file(coords_pth,box_size=0)
            for box in coordinates:
                micrograph_names.append(os.path.basename(micrographs_mapped[c_i]))
                coords_x.append(box.x)
                coords_y.append(box.y)

        data_dict = {'_rlnMicrographName': micrograph_names, '_rlnCoordinateX': coords_x, '_rlnCoordinateY': coords_y}
        data = pd.DataFrame(data_dict)

        new_sfile = star.StarFile(os.path.join(args.output,"cryosparc_pick.star"))
        new_sfile.update('', data, loop=True)
        new_sfile.write_star_file(overwrite=True)



        pass


import typing

from cryoloBM.bmtool import BMTool
import argparse
from argparse import ArgumentParser
import os
from pyStarDB import sp_pystardb as star
import pandas as pd
from glob import glob

class MakeAutopickStar(BMTool):

    def get_command_name(self) -> str:
        return "createAutopick"

    def create_parser(self, parentparser : ArgumentParser) -> ArgumentParser:
        parser_autopick = parentparser.add_parser(
            self.get_command_name(),
            help="Create Relion 4 autopick.star file. Needs to be run from the relion project directory.",
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        )

        autopick_required_group = parser_autopick.add_argument_group(
            "Required arguments",
            "Create Relion 4 autopick.star file. Needs to be run from the relion project directory.",
        )

        autopick_required_group.add_argument(
            "-m",
            "--micrographs",
            required=True,
            nargs='+',
            help="Path to micrograph files. Can be folder or a wildmask. In case of a wildmask, enclose the argument in single quotes like -c 'path/to/mics/*.mrc'.",
        )

        autopick_required_group.add_argument(
            "-c",
            "--coordinates",
            required=True,
            nargs='+',
            help="Path to STAR files. Can be folder or a wildmask. In case of a wildmask, enclose the argument in single quotes like -c 'path/to/star/*.star'.",
        )

        autopick_required_group.add_argument(
            "-o",
            "--output",
            required=True,
            help="Output folder where to write the autopick.star.",
        )

    @staticmethod
    def get_paths(paths: typing.List[str], fallback_wildcard: str):
        files = paths
        if len(paths)==1:
            if os.path.isdir(paths[0]):
                path_for_glob = os.path.join(paths[0], fallback_wildcard)
            else:
                path_for_glob = paths[0]
            files = glob(path_for_glob)
        return files

    def run(self, args):
        micrographs = MakeAutopickStar.get_paths(args.micrographs,"*.mrc")
        coordinates = MakeAutopickStar.get_paths(args.coordinates,"*.star")




        micrographs_filename = [os.path.splitext(os.path.basename(m))[0] for m in micrographs]
        coordinates_filename = [os.path.splitext(os.path.basename(m))[0] for m in coordinates]

        micrographs_mapped = []
        coordinates_mapped = []
        for m_i,m in enumerate(micrographs_filename):
            try:
                index = coordinates_filename.index(m)

                micrographs_mapped.append(micrographs[m_i])
                coordinates_mapped.append(coordinates[index])
            except ValueError:
                print("Did not found matching coordinate file for ", m)
        data_dict = {'_rlnMicrographName': micrographs_mapped, '_rlnMicrographCoordinates': coordinates_mapped}
        data = pd.DataFrame(data_dict)
        os.makedirs(args.output,exist_ok=True)
        new_sfile = star.StarFile(os.path.join(args.output,"autopick.star"))
        new_sfile.update('coordinate_files', data, loop=True)
        new_sfile.write_star_file(overwrite=True)



        pass


from cryoloBM.bmtool import BMTool
from argparse import ArgumentParser
import argparse
import cryolo.CoordsIO as io
from cryolo.utils import bbox_iou_vec_3d
from typing import List
from cryolo.utils import BoundBox
import numpy as np
import os


class CoordsCompare(BMTool):

    def get_command_name(self) -> str:
        '''
        :return: Command name
        '''
        return "cbox_compare"

    def create_parser(self, parentparser: ArgumentParser) -> ArgumentParser:
        helptxt = "Comparese two cbox files. Create two now"

        parser = parentparser.add_parser(
            self.get_command_name(),
            help=helptxt,
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        )

        req = parser.add_argument_group(
            "Arguments",
            helptxt,
        )

        req.add_argument(
            "--first",
            required=True,
            help="First CBOX file",
        )

        req.add_argument(
            "--second",
            required=True,
            help="Second CBOX file",
        )

        req.add_argument(
            "-b",
            "--boxsize",
            type=int,
            required=True,
            help="Boxsize",
        )

        req.add_argument(
            "-t",
            "--threshold",
            type=float,
            required=True,
            help="Threshold",
        )

        req.add_argument(
            "--minsize",
            type=float,
            required=True,
            help="Minimum size",
        )

        req.add_argument(
            "--refslice",
            type=float,
            default=None,
            help="reference slice",
        )

        req.add_argument(
            "--iou",
            type=float,
            default=0.6,
            help="IOU threshold",
        )

        req.add_argument(
            "-o",
            "--out",
            required=True,
            help="Output path",
        )

    def boundbox_to_array(self, boxes: List[BoundBox], new_size=None) -> np.array:
        boxes_data = np.empty(shape=(len(boxes), 6))
        for i, box in enumerate(boxes):
            boxes_data[i, 0] = box.x + box.w / 2
            boxes_data[i, 1] = box.y + box.h / 2
            boxes_data[i, 2] = box.z
            boxes_data[i, 3] = box.w
            boxes_data[i, 4] = box.h
            d = box.depth
            if np.isnan(d):
                d = box.w
                box.depth = box.w
            boxes_data[i, 5] = d


            if new_size:
                boxes_data[i, 3] = new_size
                boxes_data[i, 4] = new_size
                boxes_data[i, 5] = new_size

        return boxes_data

    def filter_boxes(self, boxes: List[BoundBox], size: float, threshold: float):
        filtered = []
        for box in boxes:

            confidence = box.c
            bsize = box.meta["est_box_size"][0]

            if np.isnan(confidence):
                confidence = 1


            if np.isnan(bsize):
                bsize = size+1


            if confidence> threshold:# and bsize > size:
                filtered.append(box)
        return filtered

    def run(self, args):
        pth_first = args.first
        pth_second = args.second
        thresh = args.threshold
        minsize = args.minsize
        pth_out = args.out
        boxsize = args.boxsize
        iou_thresh = args.iou


        first_file_boxes = io.read_cbox_boxfile(pth_first)
        second_file_boxes = io.read_cbox_boxfile(pth_second)

        print("Number of boxes in first file:", len(first_file_boxes))
        print("Number of boxes in second file:", len(second_file_boxes))
        first_file_boxes_filtered = self.filter_boxes(first_file_boxes,
                                                      size=minsize,
                                                      threshold=thresh)
        second_file_boxes_file_boxes_filtered = self.filter_boxes(second_file_boxes,
                                                                  size=minsize,
                                                                  threshold=thresh)

        print("Number of boxes in first file (filtered):", len(first_file_boxes_filtered))
        print("Number of boxes in second file (filtered):", len(second_file_boxes_file_boxes_filtered))

        frist_np = self.boundbox_to_array(first_file_boxes_filtered,
                                          new_size=boxsize)

        second_np = self.boundbox_to_array(second_file_boxes_file_boxes_filtered,
                                           new_size=boxsize)

        in_second_matched= []
        only_first = []
        both = []
        from tqdm import tqdm

        for index_first in tqdm(range(len(frist_np))):
            box_first = frist_np[index_first,:]
            boxes_i_rep = np.array([box_first] * len(second_np))
            ious = bbox_iou_vec_3d(boxes_i_rep, second_np)
            mask = ious >= iou_thresh
            found_matches = np.where(mask)
            np.any(mask)
            if np.any(mask) == False:
                only_first.append(index_first)
            else:
                both.append(index_first)
            for i in found_matches[0]:
                #print("add", i)
                in_second_matched.append(int(i))

        in_second_matched = set(in_second_matched)
        print("in sec", len(in_second_matched),len(second_np))
        only_second = [i for i in range(len(second_np)) if int(i) not in in_second_matched]

        print("both", len(both))
        both = set(both)
        print("both", len(both))

        #print("Both", both)
        #print("First:", only_first)
        #print("Second:", only_second)


        both_boxes = [first_file_boxes_filtered[i] for i in both]
        only_first = [first_file_boxes_filtered[i] for i in only_first]
        only_second_boxes = []
        for i in only_second:
            b = second_file_boxes_file_boxes_filtered[i]
            only_second_boxes.append(b)
        only_second = only_second_boxes
        #only_second = [second_file_boxes_file_boxes_filtered[i] for i in only_second]

        os.makedirs(pth_out, exist_ok=True)

        if args.refslice:
            maxdist = (boxsize*0.2)
            both_boxes = [box for box in both_boxes if np.abs(box.z-args.refslice)<maxdist]
            only_first = [box for box in only_first if np.abs(box.z - args.refslice) < maxdist]
            only_second = [box for box in only_second if np.abs(box.z - args.refslice) < maxdist]

        io.write_cbox_file(os.path.join(pth_out, "only_second.cbox"), only_second)
        io.write_cbox_file(os.path.join(pth_out,"both.cbox"), both_boxes)
        io.write_cbox_file(os.path.join(pth_out, "only_first.cbox"), only_first)



